(********************************************************************************)
(*  ocamlcore-api: Access to OCamlCore.org services                             *)
(*                                                                              *)
(*  Copyright (C) 2011, OCamlCore SARL                                          *)
(*                                                                              *)
(*  This library is free software; you can redistribute it and/or modify it     *)
(*  under the terms of the GNU Lesser General Public License as published by    *)
(*  the Free Software Foundation; either version 2.1 of the License, or (at     *)
(*  your option) any later version, with the OCaml static compilation           *)
(*  exception.                                                                  *)
(*                                                                              *)
(*  This library is distributed in the hope that it will be useful, but         *)
(*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  *)
(*  or FITNESS FOR A PARTICULAR PURPOSE. See the file COPYING for more          *)
(*  details.                                                                    *)
(*                                                                              *)
(*  You should have received a copy of the GNU Lesser General Public License    *)
(*  along with this library; if not, write to the Free Software Foundation,     *)
(*  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA               *)
(********************************************************************************)

(** General context for OCamlCore API, include DB connection
    @author Sylvain Le Gall
  *)

type t =
    [ `Live of OCASql.t
    | `Stub of int]

type conf =
    {
      (* Connection to postgres database *)
      oca_pghost:     string option;
      oca_pgport:     int option;
      oca_pguser:     string option;
      oca_pgpassword: string option;
      oca_pgdatabase: string option;
    }

let conf_of_inifile fn = 
  let inifile_opt =
    if Sys.file_exists fn then
      Some (new Inifiles.inifile fn)
    else
      None
  in
  let getval sct var f dflt =
    match inifile_opt with
      | Some inifile -> 
          begin
            try 
              f (inifile#getval sct var)
            with Inifiles.Invalid_element _ ->
              dflt
          end

      | None ->
          begin
            dflt
          end 
  in
    {
      oca_pghost     = getval "postgres" "host" (fun s -> Some s) None;
      oca_pguser     = getval "postgres" "user" (fun s -> Some s) None;
      oca_pgpassword = getval "postgres" "password" (fun s -> Some s) None;
      oca_pgdatabase = getval "postgres" "database" (fun s -> Some s) None;
      oca_pgport     = getval "postgres" "port" 
                         (fun s -> Some (int_of_string s)) None;
    }

let default_conf_ini = 
  "/etc/ocamlcore-api/conf.ini"

let create 
      ?(conf_ini=default_conf_ini)
      ?(conf_mod=(fun i -> i)) 
      () = 
  let conf = 
    conf_mod (conf_of_inifile conf_ini)
  in
    `Live (OCASql.create 
             ?host:conf.oca_pghost 
             ?port:conf.oca_pgport
             ?user:conf.oca_pguser
             ?password:conf.oca_pgpassword
             ?database:conf.oca_pgdatabase
             ())

let create_stub = 
  let stub_count = 
    ref 0
  in
    fun () -> 
      incr stub_count;
      (`Stub !stub_count), !stub_count

