(********************************************************************************)
(*  ocamlcore-api: Access to OCamlCore.org services                             *)
(*                                                                              *)
(*  Copyright (C) 2011, OCamlCore SARL                                          *)
(*                                                                              *)
(*  This library is free software; you can redistribute it and/or modify it     *)
(*  under the terms of the GNU Lesser General Public License as published by    *)
(*  the Free Software Foundation; either version 2.1 of the License, or (at     *)
(*  your option) any later version, with the OCaml static compilation           *)
(*  exception.                                                                  *)
(*                                                                              *)
(*  This library is distributed in the hope that it will be useful, but         *)
(*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  *)
(*  or FITNESS FOR A PARTICULAR PURPOSE. See the file COPYING for more          *)
(*  details.                                                                    *)
(*                                                                              *)
(*  You should have received a copy of the GNU Lesser General Public License    *)
(*  along with this library; if not, write to the Free Software Foundation,     *)
(*  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA               *)
(********************************************************************************)

(** Connection to FusionForge's postgresql
    @author Sylvain Le Gall
  *)

open Lwt

module PGOCaml = 
  PGOCaml_generic.Make 
    (struct 
       include Lwt 
       include Lwt_chan 
     end)

type t = 
    (PGOCaml.pa_pg_data PGOCaml.t) Lwt_pool.t

let () = 
  PGOCaml.verbose := 2

let create ?host ?port ?user ?password ?database () : t = 
  let connect () =
    let print_opt (fld, env, opt) =
      let v = 
        match opt with 
          | Some e -> e
          | None -> Sys.getenv env
      in
        Printf.printf "%s: '%s'\n%!" fld v
    in
      List.iter print_opt
        [
          "host", "PGHOST", host;
          "user", "PGUSER", user;
          "password", "PGPASSWORD", password;
          "database", "PGDATABASE", database;
        ];
      
       PGOCaml.connect ?host ?port ?user ?password ?database () 
  in
    Lwt_pool.create 16 connect

let query t query params = 
  Lwt_pool.use t
    (fun db -> 
       PGOCaml.prepare db ~query ()
       >>= fun () ->
       PGOCaml.execute db ~params ()
       >>= fun lst ->
       PGOCaml.close_statement db ()
       >>= fun () ->
       return lst)
