(********************************************************************************)
(*  ocamlcore-api: Access to OCamlCore.org services                             *)
(*                                                                              *)
(*  Copyright (C) 2011, OCamlCore SARL                                          *)
(*                                                                              *)
(*  This library is free software; you can redistribute it and/or modify it     *)
(*  under the terms of the GNU Lesser General Public License as published by    *)
(*  the Free Software Foundation; either version 2.1 of the License, or (at     *)
(*  your option) any later version, with the OCaml static compilation           *)
(*  exception.                                                                  *)
(*                                                                              *)
(*  This library is distributed in the hope that it will be useful, but         *)
(*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  *)
(*  or FITNESS FOR A PARTICULAR PURPOSE. See the file COPYING for more          *)
(*  details.                                                                    *)
(*                                                                              *)
(*  You should have received a copy of the GNU Lesser General Public License    *)
(*  along with this library; if not, write to the Free Software Foundation,     *)
(*  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA               *)
(********************************************************************************)

(** Web services for session
    @author Sylvain Le Gall
  *)

open Eliom_sessions
open XHTML.M
open Lwt
open OCASession
open Common
open Context
open Eliom_services
open Eliom_parameters

let get ~ctxt sp = 
  SessionLow.get ~ctxt:ctxt.web_ctxt ctxt.web_ignore_ip sp

let action_box ~ctxt ?sess sp = 
  let mk_a href ttl = 
    XHTML.M.a ~a:[a_href href] [pcdata ttl]
  in
    begin
      match sess with 
        | Some e -> 
            return e
        | None -> 
            get ~ctxt sp 
    end
    >>= fun sess ->
    return
      (div 
         ~a:[a_id "account"] 
         [
           match sess with 
             | Some _ ->
                 ul 
                   (li [mk_a (ctxt.web_logout sp) (s_ "Log Out")])
                   [li [mk_a (ctxt.web_my_account sp) (s_ "My account")]]
             | None ->
                 ul
                   (li [mk_a (ctxt.web_login sp) (s_ "Log In")])
                   [li [mk_a (ctxt.web_new_account sp) (s_ "New account")]]
         ])

let login_get ~ctxt sp =
  ctxt.web_login_get sp 

let logout_get ~ctxt sp = 
  ctxt.web_logout_get sp

let with_login ~ctxt sp uri elt =
  XHTML.M.a
    ~a:[a_href (ctxt.web_with_login sp uri)]
    elt

