
open Lwt
open XHTML.M
open Eliom_services
open Eliom_parameters
open Eliom_sessions
open Eliom_predefmod.Xhtml
open OCAAccount
open OCASession

let template ctnt =
  TemplateStub.make_page_stub
    ~page_title:"3rd party session management"
    []
    ctnt
    

(***********************************)

(** Services to handle login
  *)
let login ~ctxt base_path = 

  let login = 
    new_service 
      (base_path @ ["login"])
      (opt (string "redirect"))
      ()
  in

  let login_done =
    register_new_service
      ~path:(base_path @ ["login-done"])
      ~get_params:unit
      (fun sp () () ->
         return
           (template
              [p [pcdata "You are now logged in."]]))
  in

  let login_action_handler sp uri_opt user_id =
    let sess =
      {
        sess_hash       = String.make 16 'X';
        sess_user_id    = user_id;
        sess_time       = CalendarLib.Calendar.now ();
        sess_ip         = get_remote_inet_addr ~sp;
        sess_user_agent = get_user_agent ~sp;
      }
    in
    let cookies = 
      [Set 
         (None, 
          Some (Unix.gettimeofday () +. 3600.0), 
          cookie_name, 
          OCASession.to_cookie sess, false)]
    in
    let uri =
      match uri_opt with 
        | Some str ->
            uri_of_string str
        | None ->
            make_uri ~sp ~service:login_done ()
    in
      OCAStub.with_id ~ctxt (OCAStub.session_add sess);
      return (uri, cookies)
  in

  let login_action = 
    Eliom_predefmod.String_redirection.Cookies.register_new_post_service
      ~fallback:login
      ~post_params:(int "user_id")
      login_action_handler
  in

  let login_handler sp uri_opt () =
    let f lst uri_opt = 
      post_form
        ~sp
        ~service:login_action
        (fun user_id_nm ->
           [p
              [
               begin
                 match lst with 
                   | hd :: tl ->
                       let to_option selected 
                             {accnt_id = id; 
                              accnt_real_name = realname} = 
                         Option ([], id, Some (pcdata realname), selected)
                       in
                         int_select ~name:user_id_nm
                           (to_option true hd)
                           (List.map (to_option false) tl)
                   | [] ->
                       pcdata "no accounts"
               end;

               string_input ~input_type:`Submit ~value:"Login" ();
              ]
           ]
        ) 
        uri_opt
    in
      OCAAccount.list ~ctxt () 
      >>= fun lst -> 
      return (template [f lst uri_opt])
  in

  let login_get =
    Eliom_predefmod.Unit.Cookies.register_new_service
      ~path:["login"]
      ~get_params:(string "login" ** string "password")
      (fun sp (login, password) () ->
         OCAAccount.of_unix_name ~ctxt login
         >>= fun accnt -> 
         login_action_handler sp None accnt.accnt_id
         >>= fun (_, cookies) ->
         return ((), cookies))
  in

    register login login_handler;
    login, login_get


(***********************************)

(** Services to handle logout 
  *)
let logout ~ctxt sess_get base_path = 
  let logout =
    new_service
      (base_path @ ["logout"])
      (opt (string "redirect"))
      ()
  in

  let logout_done = 
    register_new_service 
      ~path:(base_path @ ["logout-done"])
      ~get_params:unit
      (fun sp () () ->
         return
           (template 
              [p [pcdata "You are now logged out."]]))
  in

  let logout_handler sp uri_opt () = 
    let uri = 
      match uri_opt with 
        | Some str ->
            uri_of_string str 
        | None ->
            make_uri ~sp ~service:logout_done ()
    in
      sess_get sp
      >>= 
      begin
        function
          | Some sess ->
              OCAStub.with_id ~ctxt (OCAStub.session_remove sess);
              return ()
          | None ->
              return () 
      end
      >>= fun () ->
      return (uri, [Unset (None, cookie_name)])
  in

  let logout_get =
    Eliom_predefmod.Unit.Cookies.register_new_service 
      ~path:["logout"]
      ~get_params:unit
      (fun sp () () ->
         logout_handler sp None ()
         >>= fun (_, cookies) ->
         return ((), cookies))
  in

    Eliom_predefmod.String_redirection.Cookies.register 
      logout logout_handler;
    logout, logout_get
